﻿using System;
using System.Drawing;
using System.Text;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Net.Http;
using System.IO;

namespace TeemoTool
{
    public partial class MainForm : Form
    {
        private Thread TRepeat;
        private bool active;
        private string palletPrint;
        private string host = "http://10.10.1.3:8000/";
        private string stationId;
        private string WO;
        private string oldWO;
        private string palletId;
        private string palletInput;
        private string cartonId;
        private string cartonInput;
        private const string NULL = "NULL";
        private int giftQty;
        private long timeGet;
        private static readonly HttpClient client = new HttpClient();

        public MainForm()
        {
            InitializeComponent();
            LoadConfig();
            TRepeat = new Thread(Repeat);
            TRepeat.IsBackground = true;
            TRepeat.Start();
        }

        private long time()
        {
            return DateTime.Now.Ticks / TimeSpan.TicksPerMillisecond;
        }

        private bool CheckTB()
        {
            int count = 0;
            for (int i = 0; i < IDStationTB.TextLength; i++)
            {
                if (IDStationTB.Text[i] == '_')
                {
                    count++;
                }
            }
            return count == 2;
        }

        private void Repeat()
        {
            while (true)
            {
                if (active)
                {
                    CheckCarton();
                }
                Thread.Sleep(1000);
            }
        }

        private void LoadConfig()
        {
            WO = NULL;
            palletId = NULL;
            cartonId = NULL;
            string[] config = File.ReadAllText("Config").Split('|');
            stationId = config[0];
            palletPrint = config[1];
            IDStationTB.Text = stationId;
            SendPrintCmd.Checked = palletPrint.Equals(NULL) ? false : true;
        }

        private void GetWO()
        {
            try
            {
                string[] info = stationId.Split('_');
                string payload = "{\"line_name\":\"" + info[0] + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/workoders/getLineInformation/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    Line var = JsonSerializer.Deserialize<Line>(t.Result[1]);
                    WO = var.WO_activate;
                }
            }
            catch
            {
                GetWO();
            }
        }

        private void GetPalletID()
        {
            if (WO.Equals(NULL))
            {
                GetWO();
                if (WO.Equals(NULL))
                {
                    return;
                }
            }
            try
            {
                string payload = "{\"wo\":\"" + WO + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/pallet/getPalletOpenByWO/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    GetPallet var = JsonSerializer.Deserialize<GetPallet>(t.Result[1]);
                    palletId = var.data.palletSn;
                }
            }
            catch
            {
                GetPalletID();
            }
        }

        private void CheckCarton()
        {
            if (palletId.Equals(NULL))
            {
                GetPalletID();
                if (palletId.Equals(NULL))
                {
                    return;
                }
            }
            try
            {
                string payload = "{\"wo\":\"" + WO + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/carton/getCartonOpenByWO/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    GetCarton var = JsonSerializer.Deserialize<GetCarton>(t.Result[1]);
                    string carton = var.data.cartonSn;
                    int qty = var.data.giftSn.Length;
                    if (cartonId.Equals(NULL))
                    {
                        cartonId = carton;
                        return;
                    }
                    if (!carton.Equals(cartonId))
                    {
                        cartonInput = cartonId;
                        cartonId = carton;
                        InputCarton();
                        return;
                    }
                    else
                    {
                        if (!palletInput.Equals(palletId))
                        {
                            InputCarton();
                            return;
                        }
                        if (qty != giftQty)
                        {
                            giftQty = qty;
                            timeGet = time();
                        }
                        else if (time() - timeGet > 60000)
                        {
                            palletId = NULL;
                            if (!palletPrint.Equals(NULL) && !oldWO.Equals(WO))
                            {
                                palletPrint = palletInput;
                                PrintTask.Enabled = true;
                            }
                            WO = NULL;
                        }
                    }
                }
            }
            catch
            {
                CheckCarton();
            }
        }

        private void InputCarton()
        {
            try
            {
                string[] info = stationId.Split('_');
                string payload = "{\"station_name\":\"" + info[1] + "\",\"line_id\":\"" + info[0] + "\",\"station_index\":" + info[2] + ",\"wo\":\"" + WO + "\",\"cartonSn\":\"" + cartonInput + "\",\"palletSn\":\"" + palletId + "\"}";
                var t = Task.Run(() => Post(host + "shopfloor/pallet/inputCartonToPallet/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    giftQty = -1;
                    palletInput = palletId;
                    oldWO = WO;
                }
                else
                {
                    palletId = NULL;
                    if (!palletPrint.Equals(NULL))
                    {
                        palletPrint = palletInput;
                        PrintTask.Enabled = true;
                    }
                }
            }
            catch
            {
                InputCarton();
            }
        }

        private async Task<string[]> Post(string url, string payload)
        {
            HttpContent content = new StringContent(payload, Encoding.UTF8, "application/json");
            var message = await client.PostAsync(url, content);
            var result = await message.Content.ReadAsStringAsync();
            return new string[] { message.StatusCode.ToString(), result };
        }

        private void MainForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Space)
            {
                active = !active;
                LTeemo.ForeColor = active ? Color.Green : Color.Gray;
            }
        }

        private void PrintTask_Tick(object sender, EventArgs e)
        {
            Clipboard.SetText(palletPrint);
            SendKeys.SendWait("^a");
            SendKeys.SendWait("^v");
            SendKeys.Send("~");
            PrintTask.Enabled = false;
        }

        private void IDStationTB_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                IDStationTB.SelectAll();
                if (CheckTB())
                {
                    stationId = IDStationTB.Text;
                    File.WriteAllText("Config", stationId + "|" + palletPrint);
                    MessageBox.Show("Lưu thiết lập thành công!");
                }
                else
                {
                    IDStationTB.Text = "L1_Station_1";
                    MessageBox.Show("Vui lòng nhập đúng định dạng!\r\nVí dụ: L1_Station_1");
                }
            }
        }

        private void Notify_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                Notify.Visible = false;
                this.Show();
                this.WindowState = FormWindowState.Normal;
            }
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                Notify.Visible = true;
                this.Hide();
                if (active)
                {
                    Notify.ShowBalloonTip(1000);
                }
            }
        }

        private void SendPrintCmd_Click(object sender, EventArgs e)
        {
            SendPrintCmd.Checked = !SendPrintCmd.Checked;
            if (!SendPrintCmd.Checked)
            {
                palletPrint = NULL;
            }
            else if (palletPrint.Equals(NULL))
            {
                palletPrint = "PALLET";
            }
            File.WriteAllText("Config", stationId + "|" + palletPrint);
            MessageBox.Show("Đã " + (SendPrintCmd.Checked ? "kích hoạt " : "vô hiệu ") + "gửi lệnh in tem!");
        }
    }

    public class GetCarton
    {
        public string result { get; set; }
        public string message { get; set; }
        public Carton data { get; set; }
    }

    public class Carton
    {
        public string cartonSn { get; set; }
        public string[] giftSn { get; set; }
        public string SSCC { get; set; }
    }

    public class Line
    {
        public string line_name { get; set; }
        public string status { get; set; }
        public string leader { get; set; }
        public string[] op_list { get; set; }
        public string WO_activate { get; set; }
        public int WO_producted { get; set; }
        public string WO_qty { get; set; }
        public string product_name { get; set; }
        public string SKUcode { get; set; }
        public string PO_id { get; set; }
        public int PO_producted { get; set; }
        public string PO_qty { get; set; }
    }

    public class GetPallet
    {
        public string result { get; set; }
        public string message { get; set; }
        public Pallet data { get; set; }
    }

    public class Pallet
    {
        public string palletSn { get; set; }
        public string[] cartonSn { get; set; }
        public string SSCC { get; set; }
    }
}
